@@ -1,7 +1,23 @@
 <?php
-// Dynamic routing modules
-// Copied from ivr and calleridlookup modules
-// John Fawcett Sept 2009
+//    dynroute - Dynamic Route Module for Freepbx
+//    Copyright (C) 2009-2014 John Fawcett john@voipsupport.it
+//
+//    This program is free software: you can redistribute it and/or modify
+//    it under the terms of the GNU General Public License as published by
+//    the Free Software Foundation, either version 3 of the License, or
+//    any later version.
+//
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU General Public License for more details.
+//
+//    You should have received a copy of the GNU General Public License
+//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+//    This file was originally a derived work of the freepbx ivr 
+//    and calleridlookup modules in September 2009
+
 if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }
 
 function dynroute_init() {
@@ -99,23 +115,31 @@ function dynroute_get_config($engine) {
 			if(is_array($dynroutelist)) {
 				foreach($dynroutelist as $item) {
 					$id = "dynroute-".$item['dynroute_id'];
-					$details = dynroute_get_details($item['dynroute_id']);
-					if (version_compare($version, "1.6", "lt")) {
-                                                          //Escaping MySQL query - thanks to http://www.asteriskgui.com/index.php?get=utilities-mysqlscape
-                                                          $replacements = array (
-                                                                '\\' => '\\\\',
-                                                                '"' => '\\"',
-                                                                '\'' => '\\\'',
-                                                                ' ' => '\\ ',
-                                                                ',' => '\\,',
-                                                                '(' => '\\(',
-                                                                ')' => '\\)',
-                                                                '.' => '\\.',
-                                                                '|' => '\\|'
-                                                          );
-						$query = str_replace(array_keys($replacements), array_values($replacements), $item['mysql_query']);
-					} else {
-						$query = $item['mysql_query'];
+					$details = dynroute_get_details($item['dynroute_id']);	
+					if ($item['sourcetype']=='mysql') {
+						if (version_compare($version, "1.6", "lt")) {
+		                                                  //Escaping MySQL query - thanks to http://www.asteriskgui.com/index.php?get=utilities-mysqlscape
+		                                                  $replacements = array (
+		                                                        '\\' => '\\\\',
+		                                                        '"' => '\\"',
+		                                                        '\'' => '\\\'',
+		                                                        ' ' => '\\ ',
+		                                                        ',' => '\\,',
+		                                                        '(' => '\\(',
+		                                                        ')' => '\\)',
+		                                                        '.' => '\\.',
+		                                                        '|' => '\\|'
+		                                                  );
+							$query = str_replace(array_keys($replacements), array_values($replacements), $item['mysql_query']);
+						} else {
+							$query = str_replace('"','\"',$item['mysql_query']);
+						}
+					}
+					if ($item['sourcetype']=='odbc') {
+						$query = str_replace('"','\"',$item['odbc_query']);
+					}
+					if ($item['sourcetype']=='url') {
+						$query = str_replace('"','\"',$item['url_query']);
 					}
                                         $query = str_replace('[NUMBER]', '${CALLERID(num)}', $query);
                                         $query = str_replace('[INPUT]', '${dtmfinput}', $query);
@@ -133,7 +157,7 @@ function dynroute_get_config($engine) {
 						if ($item['chan_var_name'] != '')
 							$ext->add($id, 's', '', new ext_setvar('__DYNROUTE_'.$item['chan_var_name'], '${dtmfinput}'));
                                         }
-					if ($item['mysql_host']!='')
+					if ($item['sourcetype']=='mysql' && $item['mysql_host']!='' && $item['mysql_query']!='')
 					{
 						$ext->add($id, 's', '', new ext_setvar('connid', '""'));
                                         	$ext->add($id, 's', '', new ext_mysql_connect('connid', $item['mysql_host'],  $item['mysql_username'],  $item['mysql_password'],  $item['mysql_dbname']));
@@ -146,19 +170,36 @@ function dynroute_get_config($engine) {
 							$ext->add($id, 's', '', new ext_setvar('__DYNROUTE_'.$item['chan_var_name_res'], '${dynroute}'));
 						$ext->add($id, 's', '', new ext_gotoif('$[${fetchid} = 0]',$id.',1,1'));
                                         }
-					$dests = dynroute_get_dests($item['dynroute_id']);
+					if ($item['sourcetype']=='url' && $item['url_query']!='')
+					{
+						$ext->add($id, 's', '', new ext_setvar('CURLOPT(dnstimeout)','5'));
+						$ext->add($id, 's', '', new ext_setvar('CURLOPT(conntimeout)','5'));
+						$ext->add($id, 's', '', new ext_setvar('CURLOPT(ftptimeout)','5'));
+						$ext->add($id, 's', '', new ext_setvar('CURLOPT(httptimeout)','5'));
+						$ext->add($id, 's', '', new ext_setvar('dynroute', '${CURL'.'("'.$query.'")}'));
+						$ext->add($id, 's', '', new ext_gotoif('$["${dynroute}" = ""]',$id.',1,1'));
+						if ($item['chan_var_name_res'] != '')
+							$ext->add($id, 's', '', new ext_setvar('__DYNROUTE_'.$item['chan_var_name_res'], '${dynroute}'));
+                                        }
+					if ($item['sourcetype']=='odbc' && $item['odbc_func']!='')
+					{
+						$ext->add($id, 's', '', new ext_setvar('dynroute', '${ODBC_'.$item['odbc_func'].'("'.$query.'")}'));
+						if ($item['chan_var_name_res'] != '')
+							$ext->add($id, 's', '', new ext_setvar('__DYNROUTE_'.$item['chan_var_name_res'], '${dynroute}'));
+                                        }
+					if ($item['sourcetype']=='none' && $item['enable_dtmf_input']=='CHECKED')
+                                        {
+                                                $ext->add($id, 's', '', new ext_setvar('dynroute','${dtmfinput}'));
+                                        }
+					$dests = dynroute_get_dests($item['dynroute_id'],'n');
 					if (!empty($dests)) {
-						$default_dest='';
 						foreach($dests as $dest) {
-							if ($dest['selection'] == 'default') {
-							 	$default_dest=$dest['dest'];
-							} else {
-								$ext->add($id, 's', '', new ext_gotoif('$["${dynroute}" = "'.$dest['selection'].'"]',$dest['dest']));
-							}
+							$ext->add($id, 's', '', new ext_gotoif('$["${dynroute}" = "'.$dest['selection'].'"]',$dest['dest']));
 						}
 					}
 					$ext->add($id, 's', '', new ext_goto($id.',1,1'));
-					if ($default_dest != '') $ext->add($id, '1', '', new ext_goto($default_dest));
+					$dests = dynroute_get_dests($item['dynroute_id'],'y');
+					if (!empty($dests) && $dests[0]['dest'] != '') $ext->add($id, '1', '', new ext_goto($dests[0]['dest']));
 					$ext->add($id, '1', '', new ext_hangup(''));
 				}
 			}
@@ -180,26 +221,30 @@ function dynroute_get_dynroute_id($name) {
 	
 }
 
-function dynroute_add_command($id, $cmd, $dest) {
+function dynroute_add_command($id, $cmd, $dest, $default_dest) {
 	global $db;
 	// Does it already exist?
-	$res = $db->getRow("SELECT * from dynroute_dests where dynroute_id='$id' and selection='$cmd'");
+	$res = $db->getRow("SELECT * from dynroute_dests where dynroute_id='$id' and selection='$cmd' and default_dest='$default_dest'");
 	if (count($res) == 0) {
 		// Just add it.
-		sql("INSERT INTO dynroute_dests VALUES('$id', '$cmd', '$dest')");
+		sql("INSERT INTO dynroute_dests (dynroute_id, selection, default_dest, dest) VALUES('$id', '$cmd', '$default_dest', '$dest')");
 	} else {
 		// Update it.
-		sql("UPDATE dynroute_dests SET dest='$dest' where dynroute_id='$id' and selection='$cmd'");
+		sql("UPDATE dynroute_dests SET dest='$dest' where dynroute_id='$id' and selection='$cmd' and default_dest='$default_dest'");
 	}
 }
 function dynroute_do_edit($id, $post) {
 	global $db;
         $displayname = $db->escapeSimple($post['displayname']);
+        $sourcetype = $db->escapeSimple($post['sourcetype']);
         $mysql_host = $db->escapeSimple($post['mysql_host']);
         $mysql_dbname = $db->escapeSimple($post['mysql_dbname']);
         $mysql_query = $db->escapeSimple($post['mysql_query']);
         $mysql_username = $db->escapeSimple($post['mysql_username']);
         $mysql_password = $db->escapeSimple($post['mysql_password']);
+        $odbc_func = $db->escapeSimple($post['odbc_func']);
+        $odbc_query = $db->escapeSimple($post['odbc_query']);
+        $url_query = $db->escapeSimple($post['url_query']);
         $annmsg_id = isset($post['annmsg_id'])?$post['annmsg_id']:'';
         $enable_dtmf_input = isset($post['enable_dtmf_input'])?$post['enable_dtmf_input']:'';
 
@@ -209,19 +254,21 @@ function dynroute_do_edit($id, $post) {
         $timeout = isset($post['timeout'])?$post['timeout']:'';
         $chan_var_name = isset($post['chan_var_name'])?$post['chan_var_name']:'';
         $chan_var_name_res = isset($post['chan_var_name_res'])?$post['chan_var_name_res']:'';
-
  
 	
 	$sql = "
 	UPDATE dynroute 
 	SET 
 		displayname='$displayname', 
-		sourcetype='mysql', 
+		sourcetype='$sourcetype', 
 		mysql_host='$mysql_host', 
 		mysql_dbname='$mysql_dbname', 
 		mysql_username='$mysql_username', 
 		mysql_password='$mysql_password', 
 		mysql_query='$mysql_query',
+		odbc_func='$odbc_func',
+		odbc_query='$odbc_query',
+		url_query='$url_query',
 		announcement_id='$annmsg_id',  
 		enable_dtmf_input='$enable_dtmf_input',  
 		timeout='$timeout',  
@@ -235,6 +282,8 @@ function dynroute_do_edit($id, $post) {
 	sql("DELETE FROM dynroute_dests where dynroute_id='$id'");
 	// Now, lets find all the goto's in the post. Destinations return gotoN => foo and get fooN for the dest.
 	// Is that right, or am I missing something?
+
+	$first_option=true;
 	foreach(array_keys($post) as $var) {
 		if (preg_match('/goto(\d+)/', $var, $match)) {
 			// This is a really horrible line of code. take N, and get value of fooN. See above. Note we
@@ -243,8 +292,12 @@ function dynroute_do_edit($id, $post) {
 			$cmd = $post['option'.$match[1]];
 			// Debugging if it all goes pear shaped.
 			// print "I think pushing $cmd does $dest<br>\n";
+			if ($first_option)  {
+				dynroute_add_command($id, $cmd, $dest, 'y');
+				$first_option=false;
+			}
 			if (strlen($cmd))
-				dynroute_add_command($id, $cmd, $dest);
+				dynroute_add_command($id, $cmd, $dest, 'n');
 		}
 	}
 }
@@ -272,10 +325,21 @@ function dynroute_get_details($id) {
         return $res[0];
 }
 
-function dynroute_get_dests($id) {
+function dynroute_get_dests($id,$scope) {
 	global $db;
+	switch($scope) {
+		case 'y':
+			$sql = "SELECT selection, dest FROM dynroute_dests where dynroute_id='$id' AND default_dest='y' ORDER BY selection";
+		break;
+		case 'n':
+			$sql = "SELECT selection, dest FROM dynroute_dests where dynroute_id='$id' AND default_dest='n' ORDER BY selection";
+		break;
+		case 'a':
+		default:
+			$sql = "SELECT selection, dest FROM dynroute_dests where dynroute_id='$id' ORDER BY selection";
+		break;
+	}
 
-	$sql = "SELECT selection, dest FROM dynroute_dests where dynroute_id='$id' ORDER BY selection";
         $res = $db->getAll($sql, DB_FETCHMODE_ASSOC);
         if(DB::IsError($res)) {
                 return null;
@@ -299,7 +363,7 @@ function dynroute_check_destinations($dest=true) {
 	if (is_array($dest) && empty($dest)) {
 		return $destlist;
 	}
-	$sql = "SELECT dest, displayname, selection, a.dynroute_id dynroute_id FROM dynroute a INNER JOIN dynroute_dests d ON a.dynroute_id = d.dynroute_id  ";
+	$sql = "SELECT dest, default_dest, displayname, selection, a.dynroute_id dynroute_id FROM dynroute a INNER JOIN dynroute_dests d ON a.dynroute_id = d.dynroute_id  ";
 	if ($dest !== true) {
 		$sql .= "WHERE dest in ('".implode("','",$dest)."')";
 	}
@@ -309,9 +373,10 @@ function dynroute_check_destinations($dest=true) {
 	foreach ($results as $result) {
 		$thisdest = $result['dest'];
 		$thisid   = $result['dynroute_id'];
+		if ($result['default_dest']=='y') $sel='Default'; else $sel=$result['selection'];
 		$destlist[] = array(
 			'dest' => $thisdest,
-			'description' => sprintf(_("Route: %s / Option: %s"),$result['displayname'],$result['selection']),
+			'description' => sprintf(_("Route: %s / Destination: %s"),$result['displayname'],$sel),
 			'edit_url' => 'config.php?display=dynroute&action=edit&id='.urlencode($thisid),
 		);
 	}
